function x = minres2( A,b )

%        x = minres2( A,b );
% applies 2 iterations of minres to the system Ax = b,
% where A is a symmetric matrix and we assume b = a sparse vector.

% 03 Apr 2020: First version, Michael Saunders, ICME, Stanford University.
% 27 Apr 2020: b sparse but not a unit vector.
%              Lanczos is
%                 beta1 v1 = b,                 p1 = A*v1,    alpha1 = v1'*p1
%                 beta2 v2 = p1 - alpha1*v1,    p2 = A*v2,    alpha2 = v2'*p2.
%                 beta3 v3 = p2 - alpha2*v2 - beta2*v1.
% All vectors should stay sparse.

   tolbeta = 1e-8;
   beta1   = norm(b);
   v1      = b/beta1;
   p       = A*v1;
   alpha1  = full(v1'*p);
   v2      = p - alpha1*v1;
   beta2   = norm(v2);

   if beta2 <= tolbeta
      x = b;          % Allow for singular A giving alpha1 = 0.
      if alpha1 ~= 0, x = b/alpha1; end
   else
      v2     = v2/beta2;
      p      = A*v2;
      alpha2 = full(v2'*p);
      v3     = p - alpha2*v2 - beta2*v1;
      beta3  = norm(v3);

      B3     = [alpha1   beta2
                beta2    alpha2
                  0      beta3];
      y2     = B3 \ [beta1; 0; 0];
   %  x      = [v1 v2]*y2;         % Doesn't give sparse x!
      x      = v1*y2(1) + v2*y2(2);
   end
%  keyboard
