function x = minres3( A,b )
%        x = minres3( A,b );
% applies 3 iterations of minres to the system Ax = b,
% where A is a symmetric matrix and we assume b = a sparse vector.

% 26 Apr 2020: First version of minres3.
%              Michael Saunders, ICME, Stanford University.
%              Lanczos is
%                 beta1 v1 = b,                 p1 = A*v1,    alpha1 = v1'*p1
%                 beta2 v2 = p1 - alpha1*v1,    p2 = A*v2,    alpha2 = v2'*p2.
%                 beta3 v3 = p2 - alpha2*v2 - beta2*v1.
%                 beta4 v4 = p3 - alpha3*v3 - beta3*v2.
% All vectors should stay sparse.

%%%%%%% Prototype calls minres.
   if 0
     x0     = [];
     M      = [];
     precon = 0;
     shift  = 0.0;
     show   = 0;
     check  = 1;
     itnlim = 3;
     rtol   = 1e-6;
     tolMPD = 1e-2;

%  [ x, istop, itn, rnorm, Arnorm, Anorm, Acond, xnorm, betacheck ] = ...
     x = ...
     minres( x0, A, b, M, precon, shift, show, check, itnlim, rtol, tolMPD );
     xsave = x;
   end

%% Hardwire x = minres3( A,r ).

   tolbeta = 1e-8;
   beta1   = norm(b);
   v1      = b/beta1;
   p       = A*v1;
   alpha1  = full(v1'*p);
   v2      = p - alpha1*v1;
   beta2   = norm(v2);

   if beta2 <= tolbeta
      x = b;          % Allow for singular A giving alpha1 = 0.
      if alpha1 ~= 0, x = b/alpha1; end
   else
      v2     = p/beta2;
      p      = A*v2;
      alpha2 = full(v2'*p);
      v3     = p - alpha2*v2 - beta2*v1;
      beta3  = norm(v3);

      if beta3 <= tolbeta  % 2 iterations is enough
         B3     = [alpha1   beta2
                   beta2    alpha2
                     0      beta3];
         y2     = B3 \ [beta1; 0; 0];
      %  x      = [v1 v2]*y2;         % Doesn't give sparse x!
         x      = v1*y2(1) + v2*y2(2);
      
      else % 3rd iteration
         v3     = v3/beta3;
         p      = A*v3;
         alpha3 = full(v3'*p);
         v4     = p - alpha3*v3 - beta3*v2;
         beta4  = norm(v4);

         B4     = [alpha1   beta2      0
                   beta2    alpha2   beta3
                     0      beta3    alpha3
	             0        0      beta4];
         y3     = B4 \ [beta1; 0; 0; 0];
         x      = v1*y3(1) + v2*y3(2) + v3*y3(3);
      end
   end
%  keyboard
% end function minres3
