function xMchol = minresMchol( A,b,tol,rtol );

% tol=1e-3;  rtol=1e-8;  xMchol = minresMchol( A,b,tol,rtol );
% runs minres on Ax = b with preconditioner M = Mchol(A,tol),
% which makes A2 = sparsified A by deleting nonzeros smaller than tol.
% A is normally symmetric positive-definite.
% A should be scaled first (use scaleSPD).
% minresMchol is used to test minres with
% precon=1 and M = decomposition(A2,'chol').

% Needs Mchol, minres.

% 12 May 2020: First version of minresMchol.
% Maintainers: Shaked Regev and Michael Saunders, ICME, Stanford University.
% 16 Jun 2020: Added input rtol and output xMchol.

   fprintf('\n M = Mchol( A,tol )')
   tic
   n      = length(A);
   M      = Mchol(A,tol);
   TMchol = toc;
   fprintf(' Time for Mchol =%9.2e\n', TMchol)
   if isempty(M), return, end

   x0     = [];
   precon = 1;
   shift  = 0.0;
   show   = 1;
   check  = 0;
   itnlim = min(4*n,50000);
%  rtol   = 1e-8;   % Now an input parameter
   tolMPD = 1e-4;
   facMPD = 200;
   tic

   [ xMchol, istop, itn, rnorm, Arnorm, Anorm, Acond, xnorm, betacheck ] = ...
       minres( x0, A, b, M, precon, shift, show, check, itnlim, rtol, tolMPD );
   TSMchol = toc;
   fprintf(' Time for minres with Mchol =%9.2e\n', TSMchol)

end % function minresMchol
