function xminres3 = minresMminres3( A,b,rtol );

% rtol = 1e-8;   xminres3 = minresMminres3( A,b,rtol );
% runs minres on Ax = b with preconditioner Mminres3 and accuracy tolerance rtol.

% 29 Apr 2020: First version of minresMminres3.
%              Shaked Regeve and Michael Saunders,
%              ICME, Stanford University.
% 09 May 2020: facMPD = 200 needs fewer restarts than facMPD = 100.
%              Final preconditioner is M + gammasum*I.
% 16 Jun 2020: Added input rtol and output xminres3.

   fprintf('\n M = Mminres3( A )')
   tic
   n      = length(A);
   M      = Mminres3( A );
   TMminres3 = toc;
   fprintf('\n Time for Mminres3 =%9.2e\n', TMminres3)

   x0     = [];
   shift  = 0.0;
   show   = 1;
   check  = 0;
   itnlim = min(4*n,50000);
%  rtol   = 1e-8;   % Now an input parameter
   tolMPD = 1e-4;
   facMPD = 200;
   precon = 2;      % minres handles precon = 0,1,2
   tic

   [ xminres3, istop, itn, rnorm, Arnorm, Anorm, Acond, xnorm, betacheck ] = ...
       minres( x0, A, b, M, precon, shift, show, check, itnlim, rtol, tolMPD );

   itntotal   = itn;
   gammasum   = 0;
   nrestart   = 0;
   maxrestart = 50;

   while nrestart < maxrestart & istop==9
      nrestart = nrestart + 1;
      gamma    = facMPD*(tolMPD - betacheck);
      gammasum = gammasum + gamma;
      M        = M + gamma*speye(n);
      fprintf('\n nrestart =%3i         M = M + gamma*I', nrestart)
      fprintf('\n gamma =%9.2e      gammasum =%9.2e\n', gamma,gammasum)
      x0       = xminres3;

      [ xminres3, istop, itn, rnorm, Arnorm, Anorm, Acond, xnorm, betacheck ] = ...
          minres( x0, A, b, M, precon, shift, show, check, itnlim, rtol, tolMPD );

      itntotal = itntotal + itn;
   end
   TSminres3 = toc;
   fprintf('\n nrestart =%3i    gammasum =%8.1e    itntotal =%7i\n', ...
               nrestart,        gammasum,          itntotal)
   fprintf(' Time for minres with Mminres3 =%9.2e\n', TSminres3)

end % function minresMminres3
