function xSSAI = minresSSAI( A,b,rtol );

% rtol = 1e-8;   xSSAI = minresSSAI( A,b,rtol );
% runs minres on Ax = b with preconditioner SSAI and accuracy tolerance rtol.

% 29 Apr 2020: First version of minresSSAI.
%              Shaked Regev and Michael Saunders,
%              ICME, Stanford University.
% 08 May 2020: facMPD = 200 needs fewer restarts than facMPD = 100.
%              Final preconditioner is M + gammasum*I.
% 16 Jun 2020: Input rtol and output xSSAI are helpful for testSPD( rtol ).

   fprintf('\n M = SSAI( A )')
   tic
   n      = length(A);
   lfil   = ceil(nnz(A)/n);
   M      = SSAI(A,n,lfil);   % entire_r_sparse_inverse(A,n,lfil);
   TSSAI  = toc;
   fprintf(' Time for SSAI =%9.2e\n', TSSAI)

   x0     = [];
   precon = 2;
   shift  = 0.0;
   show   = 1;
   check  = 0;
   itnlim = min(4*n,50000);
 % rtol   = 1e-8;   % Now an input parameter
   tolMPD = 1e-4;
   facMPD = 200;
   tic

   [ xSSAI, istop, itn, rnorm, Arnorm, Anorm, Acond, xnorm, betacheck ] = ...
       minres( x0, A, b, M, precon, shift, show, check, itnlim, rtol, tolMPD );

   itntotal   = itn;
   gammasum   = 0;
   nrestart   = 0;
   maxrestart = 50;

   while nrestart < maxrestart & istop==9
      nrestart = nrestart + 1;
      gamma    = facMPD*(tolMPD - betacheck);
      gammasum = gammasum + gamma;
      M        = M + gamma*speye(n);
      fprintf('\n nrestart =%3i         M = M + gamma*I', nrestart)
      fprintf('\n gamma =%9.2e      gammasum =%9.2e\n', gamma,gammasum)
      x0       = xSSAI;

      [ xSSAI, istop, itn, rnorm, Arnorm, Anorm, Acond, xnorm, betacheck ] = ...
          minres( x0, A, b, M, precon, shift, show, check, itnlim, rtol, tolMPD );

      itntotal = itntotal + itn;
   end
   TSSAI = toc;
   fprintf('\n nrestart =%3i    gammasum =%8.1e    itntotal =%7i\n', ...
               nrestart,        gammasum,          itntotal)
   fprintf(' Time for minres with SSAI =%9.2e\n', TSSAI)

end % function minresSSAI
