function [DAD,D] = scaleSPD( A,iprint )

% iprint = 1;   [DAD,D] = scaleSPD(A,iprint);
% assumes A is a sparse SPD matrix (symmetric positive definite).
% It returns DAD and diagonal sparse D such that DAD = D*A*D is
% exactly symmetric with diag(DAD) = I.
% If iprint>0, the min and max scales in D are printed.

% 06 May 2020: First version of scaleSPD.
%              Shaked Regev and Michael Saunders, ICME, Stanford University.

  [m,n] = size(A);
  if iprint
     fprintf('\n scaleSPD: Symmetric scaling of SPD matrix.   n =%9i   nnz =%9i', n,nnz(A))
  end
  if m ~= n, disp('A must be square'); return; end

  scale = full(diag(A));         % dense column vector
  Iz    = find(scale==0);
  lenz  = length(Iz);
  if lenz > 0
    disp(['A  contains ', num2str(lenz), ' empty columns'])
    scale(Iz) = 1;
  end

  scale = 1./sqrt(scale);
  D     = diag(sparse(scale));
  A     = D*tril(A,-1)*D;
  DAD   = A + A' + speye(n);

  if iprint
    [smin,jmin] = min(scale);
    [smax,jmax] = max(scale);
    fprintf('\n\n   Min scale                Max scale')
    fprintf('\n  Col %7g %9.1e    Col %7g %9.1e\n', jmin, smin, jmax, smax)
  end

end % function scaleSPD
